@@ -1,6 +1,6 @@
+require 'fugit'
 require 'sidekiq'
 require 'sidekiq/util'
-require 'rufus-scheduler'
 require 'sidekiq/cron/support'
 
 module Sidekiq
@@ -12,6 +12,7 @@ module Sidekiq
 
       #how long we would like to store informations about previous enqueues
       REMEMBER_THRESHOLD = 24 * 60 * 60
+      LAST_ENQUEUE_TIME_FORMAT = '%Y-%m-%d %H:%M:%S %z'
 
       #crucial part of whole enquing job
       def should_enque? time
@@ -46,7 +47,7 @@ module Sidekiq
 
       #enque cron job to queue
       def enque! time = Time.now.utc
-        @last_enqueue_time = time
+        @last_enqueue_time = time.strftime(LAST_ENQUEUE_TIME_FORMAT)
 
         klass_const =
             begin
@@ -55,21 +56,23 @@ module Sidekiq
               nil
             end
 
-        if klass_const
-          if defined?(ActiveJob::Base) && klass_const < ActiveJob::Base
-            enqueue_active_job(klass_const)
+        jid =
+          if klass_const
+            if defined?(ActiveJob::Base) && klass_const < ActiveJob::Base
+              enqueue_active_job(klass_const).try :provider_job_id
+            else
+              enqueue_sidekiq_worker(klass_const)
+            end
           else
-            enqueue_sidekiq_worker(klass_const)
+            if @active_job
+              Sidekiq::Client.push(active_job_message)
+            else
+              Sidekiq::Client.push(sidekiq_worker_message)
+            end
           end
-        else
-          if @active_job
-            Sidekiq::Client.push(active_job_message)
-          else
-            Sidekiq::Client.push(sidekiq_worker_message)
-          end
-        end
 
         save_last_enqueue_time
+        add_jid_history jid
         logger.debug { "enqueued #{@name}: #{@message}" }
       end
 
@@ -81,14 +84,10 @@ module Sidekiq
 
       def enqueue_active_job(klass_const)
         klass_const.set(queue: @queue).perform_later(*@args)
-
-        true
       end
 
       def enqueue_sidekiq_worker(klass_const)
         klass_const.set(queue: queue_name_with_prefix).perform_async(*@args)
-
-        true
       end
 
       # siodekiq worker message
@@ -123,6 +122,7 @@ module Sidekiq
       def active_job_message
         {
           'class'        => 'ActiveJob::QueueAdapters::SidekiqAdapter::JobWrapper',
+          'wrapped'      => @klass,
           'queue'        => @queue_name_with_prefix,
           'description'  => @description,
           'args'         => [{
@@ -273,13 +273,14 @@ module Sidekiq
 
         #set last enqueue time - from args or from existing job
         if args['last_enqueue_time'] && !args['last_enqueue_time'].empty?
-          @last_enqueue_time = Time.parse(args['last_enqueue_time'])
+          @last_enqueue_time = parse_enqueue_time(args['last_enqueue_time'])
         else
           @last_enqueue_time = last_enqueue_time_from_redis
         end
 
         #get right arguments for job
         @args = args["args"].nil? ? [] : parse_args( args["args"] )
+        @args += [Time.now.to_f] if args["date_as_argument"]
 
         @active_job = args["active_job"] == true || ("#{args["active_job"]}" =~ (/^(true|t|yes|y|1)$/i)) == 0 || false
         @active_job_queue_name_prefix = args["queue_name_prefix"]
@@ -347,6 +348,12 @@ module Sidekiq
         !enabled?
       end
 
+      def pretty_message
+        JSON.pretty_generate Sidekiq.load_json(message)
+      rescue JSON::ParserError
+        message
+      end
+
       def status_from_redis
         out = "enabled"
         if fetch_missing_args
@@ -362,12 +369,24 @@ module Sidekiq
         out = nil
         if fetch_missing_args
           Sidekiq.redis do |conn|
-            out = Time.parse(conn.hget(redis_key, "last_enqueue_time")) rescue nil
+            out = parse_enqueue_time(conn.hget(redis_key, "last_enqueue_time")) rescue nil
           end
         end
         out
       end
 
+      def jid_history_from_redis
+        out =
+          Sidekiq.redis do |conn|
+            conn.lrange(jid_history_key, 0, -1) rescue nil
+          end
+
+        # returns nil if out nil
+        out && out.map do |jid_history_raw|
+          Sidekiq.load_json jid_history_raw
+        end
+      end
+
       #export job data to hash
       def to_hash
         {
@@ -390,7 +409,7 @@ module Sidekiq
       end
 
       def valid?
-        #clear previos errors
+        #clear previous errors
         @errors = []
 
         errors << "'name' must be set" if @name.nil? || @name.size == 0
@@ -398,21 +417,15 @@ module Sidekiq
           errors << "'cron' must be set"
         else
           begin
-            cron = Rufus::Scheduler::CronLine.new(@cron)
-            cron.next_time(Time.now.utc).utc
-          rescue Exception => e
-            #fix for different versions of cron-parser
-            if e.message == "Bad Vixie-style specification bad"
-              errors << "'cron' -> #{@cron}: not a valid cronline"
-            else
-              errors << "'cron' -> #{@cron}: #{e.message}"
-            end
+            @parsed_cron = Fugit.do_parse_cron(@cron)
+          rescue => e
+            errors << "'cron' -> #{@cron.inspect} -> #{e.class}: #{e.message}"
           end
         end
 
         errors << "'klass' (or class) must be set" unless klass_valid
 
-        !errors.any?
+        errors.empty?
       end
 
       def klass_valid
@@ -460,6 +473,20 @@ module Sidekiq
         end
       end
 
+      def add_jid_history(jid)
+        jid_history = {
+          jid: jid,
+          enqueued: @last_enqueue_time
+        }
+        @history_size ||= (Sidekiq.options[:cron_history_size] || 10).to_i - 1
+        Sidekiq.redis do |conn|
+          conn.lpush jid_history_key,
+                     Sidekiq.dump_json(jid_history)
+          # keep only last 10 entries in a fifo manner
+          conn.ltrim jid_history_key, 0, @history_size
+        end
+      end
+
       # remove job from cron jobs by name
       # input:
       #   first arg: name (string) - name of job (must be same - case sensitive)
@@ -471,6 +498,9 @@ module Sidekiq
           #delete runned timestamps
           conn.del job_enqueued_key
 
+          # delete jid_history
+          conn.del jid_history_key
+
           #delete main job
           conn.del redis_key
         end
@@ -496,7 +526,7 @@ module Sidekiq
       # Parse cron specification '* * * * *' and returns
       # time when last run should be performed
       def last_time now = Time.now.utc
-        Rufus::Scheduler::CronLine.new(@cron).previous_time(now.utc).utc
+        parsed_cron.previous_time(now.utc).utc
       end
 
       def formated_enqueue_time now = Time.now.utc
@@ -525,6 +555,10 @@ module Sidekiq
 
       private
 
+      def parsed_cron
+        @parsed_cron ||= Fugit.parse_cron(@cron)
+      end
+
       def not_enqueued_after?(time)
         @last_enqueue_time.nil? || @last_enqueue_time.to_i < last_time(time).to_i
       end
@@ -550,8 +584,16 @@ module Sidekiq
         end
       end
 
+      def parse_enqueue_time(timestamp)
+        DateTime.strptime(timestamp, LAST_ENQUEUE_TIME_FORMAT).to_time.utc
+      rescue ArgumentError
+        DateTime.parse(timestamp).to_time.utc
+      end
+
       def not_past_scheduled_time?(current_time)
-        last_cron_time = Rufus::Scheduler::CronLine.new(@cron).previous_time(current_time).utc
+        last_cron_time = parsed_cron.previous_time(current_time).utc
+          # or could it be?
+        #last_cron_time = last_time(current_time)
         return false if (current_time.to_i - last_cron_time.to_i) > 60
         true
       end
@@ -577,12 +619,20 @@ module Sidekiq
         "cron_job:#{name}:enqueued"
       end
 
+      def self.jid_history_key name
+        "cron_job:#{name}:jid_history"
+      end
+
       # Redis key for storing one cron job run times
       # (when poller added job to queue)
       def job_enqueued_key
         self.class.job_enqueued_key @name
       end
 
+      def jid_history_key
+        self.class.jid_history_key @name
+      end
+
       # Give Hash
       # returns array for using it for redis.hmset
       def hash_to_redis hash
